;; Copyright (C) 1995

;; Author: Anders Stenman <stenman@isy.liu.se>
;;         Automatic Control, Linkoping University. Sweden
;;
;; Keywords: AucTeX, toolbar

;; This file is NOT part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;; Installation:
;; =============
;;
;; 1. Put the XPM icons where you want them.
;; 2. Set the variable TeX-icon-directory to the directory
;;    that holds the icons.
;; 3. Load this file.

;; ------------------------------------------------------

;; Define som useful variables

(defvar TeX-icon-directory "~/emacs/tex-toolbar/"
  "The directory where the icon data files for AucTeX are installed.")

(defvar TeX-LaTeX-icon 
  (toolbar-make-button-list 
   (expand-file-name "latex.xpm" TeX-icon-directory))
  "A LaTeX toolbar icon.")

(defvar TeX-BibTeX-icon 
  (toolbar-make-button-list 
   (expand-file-name "bibtex.xpm" TeX-icon-directory))
  "A BibTeX toolbar icon.")

(defvar TeX-view-icon 
  (toolbar-make-button-list 
   (expand-file-name "view.xpm" TeX-icon-directory))
  "A Preview toolbar icon.")

(defvar TeX-print-icon
  (toolbar-make-button-list 
   (expand-file-name "print.xpm" TeX-icon-directory))
  "A Print toolbar icon.")

(defvar TeX-file-icon
  (toolbar-make-button-list 
   (expand-file-name "file.xpm" TeX-icon-directory))
  "A File toolbar icon.")

(defvar TeX-help-icon
  (toolbar-make-button-list
   (expand-file-name "help.xpm" TeX-icon-directory))
  "A Help toolbar icon.")

;; Define the functions that we calls from the toolbar

(defun TeX-toolbar-run-LaTeX ()
  "Run LaTeX from the toolbar."
  (interactive)
  (TeX-command-menu "LaTeX"))

(defun TeX-toolbar-run-BibTeX ()
  "Run BibTeX from the toolbar."
  (interactive)
  (TeX-command-menu "BibTeX"))

(defun TeX-toolbar-preview ()
  "Run the View command from the toolbar. Automagicly inhibits
the confirmation garbage."
  (interactive)
  (let* ((entry (copy-sequence (assoc "View" TeX-command-list)))
	 TeX-command-list)
    (rplaca (nthcdr 3 entry) nil)
    (setq TeX-command-list (list entry))
    (TeX-command-menu "View")))

(defun TeX-toolbar-print ()
  (interactive)
  "Run the Print command from the toolbar. Popups a printer
selection dialog box. Automagicly inhibits the confirmation
garbage."
  (let* ((printer-list (mapcar (function car) TeX-printer-list))
	 (dialog-entry
	  (append (list "Select printer!")
		  (mapcar
		   (function
		    (lambda (pr)
		      (vector pr (list 'TeX-toolbar-print1 pr) t)))
		   printer-list)
		  (list nil
			["Cancel" ignore t]))))
    (popup-dialog-box dialog-entry)))

(defun TeX-toolbar-print1 (printer)
  (let* ((entry (copy-sequence (assoc "Print" TeX-command-list)))
	 TeX-command-list)
    (rplaca (nthcdr 3 entry) nil)
    (setq TeX-command-list (list entry))
    (TeX-command-menu-print printer TeX-print-command "Print")))
; Erik: i.p.v. TeX-print-command stond er (nth 1 printer-entry)

(defun TeX-toolbar-print-default()
  "Run the Print command from the toolbar on the default printer"
  (interactive)
  (TeX-toolbar-print1 TeX-printer-default))

(defun TeX-toolbar-file ()
  "Run the File command from the toolbar. Automagicly inhibits
the confirmation garbage."
  (interactive)
  (let* ((entry (copy-sequence (assoc "File" TeX-command-list)))
	 TeX-command-list)
    (rplaca (nthcdr 3 entry) nil)
    (setq TeX-command-list (list entry))
    (TeX-command-menu "File")))

(defun TeX-toolbar-help ()
  "Browse the AucTeX info file from the toolbar."
  (interactive)
  (info "auctex"))

;; Finally, define the toolbar.

(defvar TeX-toolbar
  '([toolbar-file-icon          find-file       t       "Open a file"   ]
    [toolbar-folder-icon        dired           t       "View directory"]
    [toolbar-disk-icon          save-buffer     t       "Save buffer"   ]
    [toolbar-cut-icon           x-kill-primary-selection t "Kill region"]
    [toolbar-copy-icon          x-copy-primary-selection t "Copy region"]
    [toolbar-paste-icon         x-yank-clipboard-selection t
                                "Paste from clipboard"]
    [toolbar-undo-icon          undo            t       "Undo edit"     ]
    [toolbar-spell-icon         toolbar-ispell  t       "Spellcheck"    ]
    [toolbar-replace-icon       query-replace   t       "Replace text"  ]
    [:style 2d :size 5]
    [TeX-LaTeX-icon             TeX-toolbar-run-LaTeX   t       
				"Run LaTeX on document"]
    [TeX-BibTeX-icon             TeX-toolbar-run-BibTeX   t       
				"Run BibTeX on document"]
    [TeX-view-icon              TeX-toolbar-preview  t  "Preview document"]
    [TeX-print-icon             TeX-toolbar-print   t   "Print document"  ]
    [TeX-file-icon              TeX-toolbar-file    t   "File document"  ]
    nil
    [TeX-help-icon              TeX-toolbar-help    t   "AucTeX help"])
  "The toolbar used in AucTeX.")

(defun TeX-install-toolbar ()
  (set-specifier top-toolbar (cons (current-buffer) TeX-toolbar)))

;;; add this line in your personal files
(add-hook 'LaTeX-mode-hook 'TeX-install-toolbar)

(provide 'tex-toolbar)

