;;; **********************************************************************
;;; Copyright (C) 1997 Teragram Corporation
;;; All rights reserved
;;;
;;; TeraSpell 97 for Emacs version 1.0.1
;;; August 16 1997
;;;
;;; teraspell.el
;;;
;;; **********************************************************************


;; xemacsp
;;  return t if xemacs is running
;;         nil otherwise

(defun xemacsp()
  (string-match "XEmacs+" (emacs-version)))

;-----------------------------------------------------------------------------;
;        emacs-version-number()                                               ;
;                                                                             ;
;        Return a list of three numbers                                       ;
;         Major version number                                                ;
;         Minor version number                                                ;
;         Subminor version number                                             ;
;                                                                             ;
;        A value of zero for any of the above version indicates that this     ;
;        version is not specified.                                            ;
;-----------------------------------------------------------------------------;
(defun emacs-version-number()
  (let ((version)
	(start)
	(main "0")
	(sub "0")
	(subsub "1")
	(end))
    (setq version (emacs-version))


    (cond ((setq start (string-match "[0-9]+\.[0-9]+\.[0-9]+" version))
	   (setq end (match-end 0))
	   (setq version (substring version start end ))

	   ;; main version number
	   (setq start (string-match "[0-9]+" version))
	   (setq end (match-end 0))
	   (setq main (substring version start end ))
	   (setq version (substring version end ))

	   ;; sub version number
	   (setq start (string-match "[0-9]+" version))
	   (setq end (match-end 0))
	   (setq sub (substring version start end ))
	   (setq version (substring version end))

	   ;; subsub version number
	   (setq start (string-match "[0-9]+" version))
	   (setq end (match-end 0))
	   (setq subsub (substring version start end )))

	  ((setq start (string-match "[0-9]+\.[0-9]+" version))
	   (setq end (match-end 0))
	   (setq version (substring version start end ))

	   ;; main version number
	   (setq start (string-match "[0-9]+" version))
	   (setq end (match-end 0))
	   (setq main (substring version start end ))
	   (setq version (substring version end))

	   ;; sub version number
	   (setq start (string-match "[0-9]+" version))
	   (setq end (match-end 0))
	   (setq sub (substring version start end ))))


    (list (car (read-from-string main))
	  (car (read-from-string sub))
	  (car (read-from-string subsub)))))

(defun teraspell-lisp-file ()
  (if (xemacsp)
      "teraspell-xemacs-19"
    (teraspell-emacs-lisp-file)))

(defun teraspell-emacs-lisp-file ()
  (let* (
	 ;; this should be sorted
	(versions
	 '(
	   (19 29 1)
	   (19 30 1)
	   (19 31 1)
	   (19 32 1)
	   (19 33 1)
	   (19 34 1)
	   ))
	(i 0)
	(ln (length versions))
	(emacs-versions (emacs-version-number))
	(n-emacs (+
		  (*  1000000 (nth 0 emacs-versions))
		  (*  1000 (nth 1 emacs-versions))
		  (nth 2 emacs-versions)))
	(current (nth i versions))
	(n (+ (* 1000000 (nth 0 current))
	      (* 1000 (nth 1 current))
	      (nth 2 current)))
	(found nil)
	(res nil))

    (while  (and (< i ln) 
		 (equal found nil))

      (setq current (nth i versions))
      (setq n (+ (* 1000000 (nth 0 current))
		 (* 1000 (nth 1 current))
		 (nth 2 current)))
      (cond ((equal n n-emacs)
	     (setq found i))
	    ((and (> n n-emacs)
		  (equal i 0))
	     (setq found -1))
	    ((and (< n n-emacs)
		  (equal i (- ln 1)))
	     (setq found i)
	     (setq i (+ i 1)))
	    ((< n n-emacs)
	     (setq i (+ i 1)))
	    ((> n n-emacs)
	     (setq found (- i 1)))))

    (cond ((< found 0)
	     (error "The version of Emacs is too old to work with TeraSpell"))
	   ((numberp found)
	    (setq res (nth  found versions)))
	   ((null found)
	    (setq res (nth  0 versions))))
    (format "teraspell-%s-%s-%s" (nth 0 res) (nth 1 res) (nth 2 res))))

(load (concat ts-dir "lisp/" (teraspell-lisp-file)))
