;;;; xslide-data.el --- XSL IDE element and attribute data
;; $Id: xslide-data.el,v 1.1 1998/09/21 13:59:58 tkg Exp $

;; Copyright (C) 1998 Tony Graham

;; Author: Tony Graham <tgraham@mulberrytech.com>

;;; This file is not part of GNU Emacs.

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


;;;; Commentary:

;; Data about elements and attributes in XSL stylesheets collected
;; in one place

;; Send bugs to xslide-bug@menteith.com
;; Use `xsl-submit-bug-report' for bug reports


;;;; Variables

(defvar xsl-xsl-ns-prefix "xsl"
  "*Prefix for the XSL namespace")

(defvar xsl-fo-ns-prefix "fo"
  "*Prefix for the Formatting Object namespace")

(defvar xsl-element-symbol-alist
  (list
   '("arg"
     "empty"
     ("name" "value")
     "xa")
   '("attribute-set"
     "empty"
     ("use")
     "xas")
   '("choose"
     "block"
     ("xml:space")
     "xc")
   '("contents"
     "empty"
     ()
     "xco")
   '("define-attribute-set"
     "block"
     ("name")
     "xdas")
   '("define-constant"
     "empty"
     ("name" "value")
     "xdc")
   '("define-macro"
     "block"
     ("name" "xml:space")
     "xdm")
   '("for-each"
     "block"
     ("select" "xml:space")
     "xfe")
   '("id"
     "empty"
     ("attribute" "element")
     "xid")
   '("if"
     "block"
     ("test" "xml:space")
     "xif")
   '("import"
     "empty"
     ("href")
     "xim")
   '("include"
     "empty"
     ("href")
     "xinc")
   '("invoke"
     "block"
     ("macro" "xml:space")
     "xinv")
   '("macro-arg"
     "empty"
     ("name" "default")
     "xma")
   '("number"
     "empty"
     ("level" "count" "from" "format" "xml:lang" "letter-value"
      "digit-group-sep" "n-digits-per-group" "sequence-src")
     "xn")
   '("otherwise"
     "block"
     ("xml:space")
     "xo")
   '("process"
     "empty"
     ("select")
     "xp")
   '("process-children"
     "empty"
     ()
     "xpc")
   '("preserve-space"
     "empty"
     ("element")
     "xps")
   '("strip-space"
     "empty"
     ("element")
     "xss")
   '("stylesheet"
     "block"
     ("result-ns" "default-space" "ident-result" "id" "xmlns:xsl"
      "xmlns:fo" "xml:space")
     "xs")
   '("template"
     "block"
     ("match" "priority" "xml:space")
     "xt")
   '("text"
     "inline"
     ("xml:space")
     "xtxt")
   '("value-of"
     "empty"
     ("expr")
     "xvo")
   '("when"
     "block"
     ("test" "xml:space")
     "xw")))

(defvar xsl-attributes-alist
  (list
   '("attribute" "att" ())
   '("count" "cnt" ())
   '("default" "def" ())
   '("default-space" "ds" ())
   '("digit-group-sep" "dgs" ())
   '("element" "ele" ())
   '("expr" "ex" ())
   '("format" "fmt" ())
   '("from" "fr" ())
   '("href" "href" ())
   '("indent-result" "ir" ())
   '("letter-value" "lv" ())
   '("level" "lvl" ())
   '("match" "m" ())
   '("name" "n" ())
   '("n-digits-per-group" "ndpg" ())
   '("priority" "p" ())
   '("result-ns" "rns" ())
   '("select" "s" ())
   '("sequence-src" "ssrc" ())
   '("test" "t" ())
   '("value" "v" ())
   '("xml:lang" "xl" ())
   '("xmlns:fo" "xnf" ())
   '("xmlns:xsl" "xnx" ())
   '("xsl-use" "xu" ())))

(defvar xsl-fo-symbol-alist
  (list
   '("block" "block" () "fb")
   '("block-level-box" "block" () "fblb")
   '("character" "block" () "fc")
   '("graphic" "block" () "fg")
   '("inline-box" "block" () "fib")
   '("link-end-locator" "block" () "flel")
   '("list-item" "block" () "fli")
   '("list-item-body" "block" () "flib")
   '("list-item-label" "block" () "flil")
   '("link" "block" () "flnk")
   '("list" "block" () "fl")
   '("page-number" "block" () "fpn")
   '("page-sequence" "block" () "fps")
   '("queue" "block" () "fq")
   '("rule-graphic" "block" () "frg")
   '("sequence" "block" () "fs")
   '("simple-page-master" "block" () "fspm")))

(defvar xsl-fo-attribute-symbol-alist-1
  (list
   '("background-attachment" "batt")
   '("background-color" "bc")
   '("background-image" "bi")
   '("background-position-x" "bpx")
   '("background-position-y" "bpy")
   '("background-repeat" "br")
   '("block-justification-letterspace-max-add" "bjlma")
   '("block-justification-letterspace-max-remove" "bjlmr")
   '("block-justification-wordspace-max" "bjwmax")
   '("block-justification-wordspace-min" "bjwmin")
   '("block-line-breaking" "blb")
   '("body-overflow" "bo")
   '("body-writing-mode" "bwm")
   '("break-after" "ba")
   '("break-before" "bb")
   '("char" "ch")
   '("char-kern" "ck")
   '("char-kern-mode" "ckm")
   '("char-ligature" "cl")
   '("color" "co")
   '("contents-alignment" "ca")
   '("contents-rotation" "cr")
   '("destination" "d")
   '("direction-embedded-text" "det")
   '("end-side-overflow" "eso")
   '("end-side-separation" "essep")
   '("end-side-size" "ess")
   '("end-side-writing-mode" "eswm")
   '("external-graphic-id" "egi")
   '("font-family" "ff")
   '("font-size" "fsi")
   '("font-size-adjust" "fsa")
   '("font-stretch" "fstr")
   '("font-style" "fs")
   '("font-style-math" "fsm")
   '("font-variant" "fv")
   '("font-weight" "fw")
   '("footer-overflow" "fo")
   '("footer-precedence" "fp")
   '("footer-separation" "fsep")
   '("footer-size" "fsi")
   '("footer-writing-mode" "fwm")
   '("graphic-line-offset" "glo")
   '("graphic-line-thickness" "glt")
   '("graphic-max-height" "gmh")
   '("graphic-max-width" "gmw")
   '("graphic-notation-id" "gni")
   '("graphic-scale" "gs")
   '("header-overflow" "ho")
   '("header-precedence" "hp")
   '("header-separation" "hsep")
   '("header-size" "hs")
   '("header-writing-mode" "hwm")
   '("height" "h")
   '("id" "id")
   '("indent-end" "ie")
   '("indent-first-line-start" "ifls")
   '("indent-start" "is")
   '("inhibit-wrap" "iwr")
   '("inline" "in")
   '("input-record-end-ignore" "irei")
   '("input-tab" "it")
   '("input-tab-expand" "ite")
   '("input-whitespace" "iw")
   '("input-whitespace-treatment" "iwt")))

(defvar xsl-fo-attribute-symbol-alist-2
  (list
   '("keep" "k")
   '("keep-with-next" "kwn")
   '("keep-with-previous" "kwp")
   '("label-alignment" "la")
   '("label-width" "lw")
   '("language" "lang")
   '("letterspacing-after-maximum" "lamax")
   '("letterspacing-after-minimum" "lamin")
   '("letterspacing-after-optimum" "lao")
   '("linespacing" "l")
   '("margin-bottom" "mb")
   '("margin-end" "me")
   '("margin-left" "ml")
   '("margin-right" "mr")
   '("margin-top" "mt")
   '("master-name" "mn")
   '("orphans" "o")
   '("overflow" "ov")
   '("page-height" "ph")
   '("page-width" "pw")
   '("page-writing-mode" "pwm")
   '("position-point-shift" "pps")
   '("position-point-x" "ppx")
   '("position-point-y" "ppy")
   '("queue-name" "qn")
   '("rule-grahic-orientation" "rgo")
   '("rule-graphic-length" "rgl")
   '("scale" "sc")
   '("score-spaces" "ss")
   '("space-after-maximum" "samax")
   '("space-after-minimum" "samin")
   '("space-after-optimum" "sao")
   '("space-before-maximum" "sbmax")
   '("space-before-minimum" "sbmin")
   '("space-before-optimum" "sbo")
   '("start-side-overflow" "sso")
   '("start-side-separation" "ssep")
   '("start-side-size" "ssz")
   '("start-side-writing-mode" "sswm")
   '("text-align" "ta")
   '("text-align-last" "tal")
   '("widows" "wdw")
   '("width" "w")
   '("wordspacing-maximum" "wmax")
   '("wordspacing-minimum" "wmin")
   '("wordspacing-optimum" "wo")
   '("writing-mode" "wm")))

(setq xsl-all-attribute-alist
      (sort
       (append
	xsl-attributes-alist
	xsl-fo-attribute-symbol-alist-1
	xsl-fo-attribute-symbol-alist-2)
       (lambda (a b) (string< (car a) (car b)))))

(setq xsl-all-elements-alist
      (sort
       (append
	(mapcar (lambda (x)
		  (cons (if xsl-xsl-ns-prefix
			    (concat xsl-xsl-ns-prefix ":" (car x))
			  (car x))
			(cdr x)))
		xsl-element-symbol-alist)
	(mapcar (lambda (x)
		  (if xsl-fo-ns-prefix
		      (cons
		       (concat xsl-fo-ns-prefix ":" (car x))
		       (cdr x))
		    x))
		xsl-fo-symbol-alist))
       (lambda (a b) (string< (car a) (car b)))))

(provide 'xslide-data)

;; end of xslide-data.el
