;;;; xslide-process.el --- Process an XSL stylesheet
;; $Id: xslide-process.el,v 1.1 1998/09/21 14:00:58 tkg Exp $

;; Copyright (C) 1998 Tony Graham

;; Author: Tony Graham <tgraham@mulberrytech.com>

;;; This file is not part of GNU Emacs.

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


;;;; Commentary:

;; Copied almost wholesale from psgml.el by Lennart Staflin

;; Send bugs to xslide-bug@menteith.com
;; Use `xsl-submit-bug-report' for bug reports


;;;; Variables:

(defvar xsl-offer-save t
  "*If non-nil, ask about saving modified buffers before \\[xsl-process] is run.")

(defvar xsl-process-command "java com.jclark.xsl.Driver %s %s %s"
  "*The shell command to process an XSL document.

This is a `format' control string that by default should contain two
`%s' conversion specifications: the first will be replaced by the
value of `xsl-xml-source' \(or the empty string, if nil\); the
second will be replaced by the current buffer's file name \(or the
empty string, if nil\).

If `xsl-process-files' is non-nil, the format string should contain
one `%s' conversion specification for each element of its result.

If xsl-process-command is a list, then every element should be a
string.  The strings will be tried in order and %-sequences in the
string will be replaced according to the list below, if the string contains
%-sequences with no replacement value the next string will be tried.

%b means the visited file of the current buffer
%s means the XSL declaration specified in the xsl-xml-source variable
%d means the file containing the DOCTYPE declaration, if not in the buffer 
")

(defvar xsl-process-files nil
  "If non-nil, a function of no arguments that returns a list of file names.
These file names will serve as the arguments to the `xsl-process-command'
format control string instead of the defaults.")

(defvar xsl-process-error-regexps
  '(("file:\\([^:]+\\):\\([0-9]+\\):\\([0-9]+\\):" 1 2 3)
    ("file:/\\(\\([A-Za-z]:\\)?[^:]+\\):\\([0-9]+\\):\\([0-9]+\\):" 1 3 4))
  "Alist of regexps to recognize error messages from `xsl-process'.
See `compilation-error-regexp-alist'.")

(defvar xsl-xml-source nil
  "*If non-nil, this is the name of the XML source file.")
(put 'xsl-xml-source 'xsl-type 'string)

(defvar xsl-xsl-result nil
  "*If non-nil, this is the name of the XSL result file.")
(put 'xsl-xsl-result 'xsl-type 'string)

(defvar xsl-process-command-history nil
  "The minibuffer history list for `xsl-process''s COMMAND argument.")

(eval-and-compile
  (autoload 'compile-internal "compile" ""))

(defun xsl-default-process-command ()
  (apply 'format xsl-process-command
	 (if xsl-process-files
	     (funcall xsl-process-files)
	   (list (or xsl-xml-source "")
		 (let ((name (buffer-file-name)))
		   (if name
		       (file-name-nondirectory name)
		     ""))
		 (or xsl-xsl-result "")))))

(defun xsl-process (command)
  "Process an XSL stylesheet.
Runs COMMAND, a shell command, in a separate process asynchronously
with output going to the buffer *XSL process*.
You can then use the command \\[next-error] to find the next error message
and move to the line in the XSL document that caused it."
  (interactive
   (list (read-from-minibuffer "Process command: "
			       (xsl-default-process-command)
			       nil nil 'xsl-process-command-history)))
  (if xsl-offer-save
      (save-some-buffers nil nil))
  (compile-internal command "No more errors" "XSL process"
		    nil
		    xsl-process-error-regexps))


(provide 'xslide-process)
