;;;; xslide-font.el --- Tony's XSL font lock keywords
;; $Id: xslide-font.el,v 1.1 1998/09/21 14:00:32 tkg Exp $

;; Copyright (C) 1998 Tony Graham

;; Author: Tony Graham <tgraham@mulberrytech.com>

;;; This file is not part of GNU Emacs.

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


;;;; Commentary:

;; Font lock mode keywords for use when editing XSL stylesheets

;; Send bugs to xslide-bug@menteith.com


;;;; Variables

;;(setq font-lock-face-attributes
;;      (append
;;       (list
;;	'(xsl-xsl-element-face "Khaki" nil nil nil)
;;	'(xsl-xsl-attribute-face "LightGray"))
;;       font-lock-face-attributes))

(defvar xsl-font-lock-face-attributes
  (list
   '(xsl-xsl-main-face "Blue")
   '(xsl-xsl-alternate-face "red")
   '(xsl-fo-main-face "PaleGreen")
   '(xsl-fo-alternate-face "YellowGreen")
   '(xsl-other-element-face "MediumSeaGreen"))
  "*List of XSL-specific font lock faces and their attributes")

;; Something to do when we read this file
(while (car xsl-font-lock-face-attributes)
  (let ((face-attributes (car xsl-font-lock-face-attributes)))
    (setq xsl-font-lock-face-attributes
	  (cdr xsl-font-lock-face-attributes))
    (font-lock-make-face face-attributes)))

;;;; Constants

(defconst xsl-elements
  '("arg" "attribute-set" "choose" "contents"
    "define-attribute-set" "define-constant" "define-macro"
    "for-each"
    "id" "if" "import" "include" "invoke"
    "macro-arg" "number" "otherwise" 
    "counters" "counter-increment" "apply-templates"
    "process" "process-children" "preserve-space"
    "strip-space" "stylesheet" "template" "text" "value-of" "when"
    "html" "table" "tr" "td" "th" "img" "meta"  "a" "p" "body"
))

(defconst xsl-attributes
  '("attribute" "count" "default" "default-space" "digit-group-sep"
    "element" "expr" "format" "from" "href" "indent-result"
    "letter-value" "level" "match" "name" "n-digits-per-group"
    "priority" "result-ns" "select" "sequence-src"
    "test" "value"
    "size" "FACE" "width" "height" "src" "height" "valign" "align" "colspan"
    "rowspan" "href"
    "xml:lang" "xmlns:fo" "xmlns:xsl" "xsl-use"))

(defconst xsl-formatting-objects
  '("block" "block-level-box" "character"
    "graphic" "inline-box" "link" "link-end-locator"
    "list-item-label" "list-item-body" "list-item" "list"
    "page-number" "page-sequence" "queue" "rule-graphic"
    "sequence" "simple-page-master"))

(defconst xsl-formatting-object-properties-1
  '("background-attachment"
    "background-color" "background-image"
    "background-position-x"
    "background-position-y"
    "background-repeat"
    "block-justification-letterspace-max-add"
    "block-justification-letterspace-max-remove"
    "block-justification-wordspace-max"
    "block-justification-wordspace-min"
    "block-line-breaking" "body-overflow"
    "body-writing-mode" "break-after"
    "break-before" "char" "char-kern"
    "char-kern-mode" "char-ligature" "color"
    "contents-alignment" "contents-rotation"
    "destination" "direction-embedded-text"
    "end-side-overflow" "end-side-separation"
    "end-side-size" "end-side-writing-mode"
    "external-graphic-id" "font-family"
    "font-size" "font-size-adjust"
    "font-stretch" "font-style"
    "font-style-math" "font-variant"
    "font-weight" "footer-overflow"
    "footer-precedence" "footer-separation"
    "footer-size" "footer-writing-mode"))

(defconst xsl-formatting-object-properties-2
  '("graphic-line-offset"
    "graphic-line-thickness"
    "graphic-max-height" "graphic-max-width"
    "graphic-notation-id" "graphic-scale"
    "header-overflow" "header-precedence"
    "header-separation" "header-size"
    "header-writing-mode" "height" "id"
    "indent-end" "indent-first-line-start"
    "indent-start" "inhibit-wrap" "inline"
    "input-record-end-ignore" "input-tab"
    "input-tab-expand" "input-whitespace"
    "input-whitespace-treatment" "keep"
    "keep-with-next" "keep-with-previous"
    "label-alignment" "label-width" "language"
    "letterspacing-after-maximum"
    "letterspacing-after-minimum"
    "letterspacing-after-optimum"
    "linespacing" "margin-bottom" "margin-end"
    "margin-left" "margin-right" "margin-top"
    "master-name" "orphans" "overflow"))

(defconst xsl-formatting-object-properties-3
  '("page-height" "page-width"
    "page-writing-mode" "position-point-shift"
    "position-point-x" "position-point-y"
    "queue-name" "rule-grahic-orientation"
    "rule-graphic-length" "scale"
    "score-spaces" "space-after-maximum"
    "space-after-minimum"
    "space-after-optimum"
    "space-before-maximum"
    "space-before-minimum"
    "space-before-optimum"
    "start-side-overflow"
    "start-side-separation" "start-side-size"
    "start-side-writing-mode" "text-align"
    "text-align-last" "widows" "width"
    "wordspacing-maximum"
    "wordspacing-minimum"
    "wordspacing-optimum" "writing-mode"))

(defvar xsl-font-lock-keywords
  (list
   ;;
   ;; Put comment patterns first so they mask anything
   ;; that might be inside the comment
   ;;
   '("<!--[^-]*\\(-[^-]+\\)*-->" . font-lock-comment-face)
   ;;
   ;; Reserved XML Processing instruction
   ;;
   '("\\(<\\?\\)\\(xml\\)\\([ \t]+\\([^?]\\|\\?[^>]\\)*\\)?\\(\\?>\\)"
     (1 font-lock-keyword-face)
     (2 font-lock-type-face)
     (5 font-lock-keyword-face))
   ;;
   ;; Non-reserved XML Processing instruction
   ;; Any XML PI that doesn't start with "<?xml"
   ;;
   '("\\(<\\?\\)\\([^ \t?>]+\\)[ \t]*\\([^?>]\\|\\?[^>]\\|>[^\n\r]\\)*\\(\\?>\\)"
     (1 font-lock-keyword-face)
     (2 font-lock-variable-name-face)
     (4 font-lock-keyword-face))
   ;;
   ;; Entity references
   ;; These come early so entity references as the names in element, etc.
   ;; declarations retain their colour and don't get turned into
   ;; font-lock-variable-name-face.  E.g:
   ;; <!ENTITY % %entity; "..." >
   ;;
   '("[%&][^; \t]+;" . font-lock-reference-face)
   ;;
   ;; Marked section start
   ;;
   '("\\(<!\\[\\)[^[]*\\(\\[\\)"
     (1 font-lock-keyword-face)
     (2 font-lock-keyword-face))
   ;;
   ;; Text inside <xsl:text>
   (list
    (concat "<" xsl-xsl-ns-prefix ":text>"
	    "\\([^<]*\\)"
	    "</" xsl-xsl-ns-prefix ":text>")
    '(1 font-lock-string-face append))
   ;;
   ;; "Other" tags
   ;;
   (list
    (concat "\\(</?\\([^xf/\?!]\\|x[^s]\\|xs[^l]\\|xsl[^:]\\|f[^o]\\|fo[^:]\\)\\([^</>]\\|/[^>]\\)*/?>\\)")
    '(1 xsl-other-element-face t))
   ;;
   ;; Content of tags
   ;;
   (list
    (concat ">\\([^<]+\\)<")
    '(1 font-lock-string-face keep))
   ;;
   ;; XSL elements
   ;;
   (list
    (concat "\\(</?\\)\\(" xsl-xsl-ns-prefix ":\\)\\("
	;;    (make-regexp xsl-elements)
	    (make-regexp
	     (mapcar 'car xsl-element-symbol-alist))
	    "\\)\\(\\s-+\\([^/>]\\|/[^>]\\)+\\)*\\(/?>\\)")
    '(1 xsl-xsl-main-face)
    '(2 xsl-xsl-alternate-face)
    '(3 xsl-xsl-main-face))
   (list
    (concat "</?" xsl-xsl-ns-prefix ":\\([^/>]\\|/[^>]\\)*\\(/?>\\)")
    '(2 xsl-xsl-main-face))
   ;;
   ;; XSL attributes
   ;;
;;   (list
;;    (concat "\\b\\("
;;	    (make-regexp xsl-attributes)
;;	    "\\)[ \t]*=\"[^\"]*\"")
;;    '(1 font-lock-keyword-face))
   (make-regexps "\\b"
		 (list (mapcar 'car xsl-attributes-alist)
		       xsl-xsl-alternate-face)
		 "[ \t]*"
		 '(("=[ \t]*\"") xsl-xsl-alternate-face)
		 '("\\([^\"<>]*\\)" 1 font-lock-variable-name-face)
		 '(("\"") xsl-xsl-alternate-face))
   (make-regexps "\\b"
		 (list (mapcar 'car xsl-attributes-alist)
		       xsl-xsl-alternate-face)
		 "[ \t]*"
		 '(("=[ \t]*'") xsl-xsl-alternate-face)
		 '("\\([^'<>]*\\)" 1 font-lock-variable-name-face)
		 '(("'") xsl-xsl-alternate-face))
   ;;
   ;; XSL formatting objects
   ;;
   (list
    (concat "\\(</?\\)\\(" xsl-fo-ns-prefix ":\\)\\("
	    (make-regexp
	     (mapcar 'car xsl-fo-symbol-alist))
	    "\\)\\(\\s-+\\([^/>]\\|/[^>]\\)+\\)*\\(/?>\\)")
    '(1 xsl-fo-main-face)
    '(2 xsl-fo-alternate-face)
    '(3 xsl-fo-main-face))
   (list
    (concat "</?" xsl-fo-ns-prefix ":\\([^/>]\\|/[^>]\\)*\\(/?>\\)")
    '(2 xsl-fo-main-face))
   ;;
   ;; XSL formatting object properties
   ;;
   (make-regexps "\\b"
		 (list (mapcar 'car xsl-fo-attribute-symbol-alist-1)
		       xsl-fo-alternate-face)
		 "[ \t]*"
		 '(("=[ \t]*\"") xsl-fo-alternate-face)
		 '("\\([^\"<>]*\\)" 1 font-lock-variable-name-face)
		 '(("\"") xsl-fo-alternate-face))
   (make-regexps "\\b"
		 (list (mapcar 'car xsl-fo-attribute-symbol-alist-1)
		       xsl-fo-alternate-face)
		 "[ \t]*"
		 '(("=[ \t]*'") xsl-fo-alternate-face)
		 '("\\([^'<>]*\\)" 1 font-lock-variable-name-face)
		 '(("'") xsl-fo-alternate-face))
   (make-regexps "\\b"
		 (list (mapcar 'car xsl-fo-attribute-symbol-alist-2)
		       xsl-fo-alternate-face)
		 "[ \t]*"
		 '(("=[ \t]*\"") xsl-fo-alternate-face)
		 '("\\([^\"<>]*\\)" 1 font-lock-variable-name-face)
		 '(("\"") xsl-fo-alternate-face))
   (make-regexps "\\b"
		 (list (mapcar 'car xsl-fo-attribute-symbol-alist-2)
		       xsl-fo-alternate-face)
		 "[ \t]*"
		 '(("=[ \t]*'") xsl-fo-alternate-face)
		 '("\\([^'<>]*\\)" 1 font-lock-variable-name-face)
		 '(("'") xsl-fo-alternate-face))
   ;;
   ;; Mark the start and end of literals, but don't do anything to their
   ;; contents
   ;;
   '("\\('\\)[^']*\\('\\)"
     (1 font-lock-string-face)
     (2 font-lock-string-face))
   '("\\(\"\\)[^\"]*\\(\"\\)"
     (1 font-lock-string-face)
     (2 font-lock-string-face))
   ;;
   ;; { } in attribute values
   ;;
   '("\\('\\|\"\\)\\({[^\\1}]*}\\)\\(\\1\\)"
     (2 font-lock-variable-name-face t))
   )
  "Additional expressions to highlight in XSL mode.")

;;;; Code:
(defun xsl-font-lock-mark-block-function ()
  "Function to mark the area of text we want to fontify.

Used with font-lock-fontify-block.  Set font-lock-mark-block-function
to this function for this function to take effect.

This function marks the area beginning five \"<\" before point and five
\">\" at ends of lines after point.  The default without a function like
this is to fontify 16 lines before and after point, but then the region
often starts or ends partway through a comment or declaration, turning
that half white because the keywords didn't match, and it just looks so
ugly."
  (let ((current-point (point)))
    (re-search-forward ">$" (point-max) 'limit 5)
    (set-mark (point))
    (goto-char current-point)
    (re-search-backward "^<" (point-min) 'limit 5)))

(defun xsl-font-lock-region-point-min ()
  "Return the start point of the region we want to fontify"
  (save-excursion
    (re-search-backward "^<" (point-min) 'limit 5)
    (point)))

(defun xsl-font-lock-region-point-max ()
  "Return the start point of the region we want to fontify"
  (save-excursion
    (re-search-forward ">$" (point-max) 'limit 5)
    (point)))

(defun xsl-font-make-faces-1 ()
  "Add XSL-specific faces to font-lock-face-attributes."
  (interactive)
  (while (car xsl-font-lock-face-attributes)
    (let ((face-attributes (car xsl-font-lock-face-attributes)))
      (setq xsl-font-lock-face-attributes
	    (cdr xsl-font-lock-face-attributes))
      (font-lock-make-face face-attributes))))

(defun xsl-font-make-faces ()
  "Add XSL-specific faces to font-lock-face-attributes."
  (interactive)
  (setq font-lock-face-attributes
	(append
	 xsl-font-lock-face-attributes
	 font-lock-face-attributes)))

(provide 'xslide-font)

;; end of xslide-font.el
